CREATE VIEW COG_CONFLICT_PATHS   ( conflict_id, parent_entity_id, child_entity_id, degree_of_sep, match_id,
  mm_rule_id, matched_entity_id, inbound_entity_id, child_event_alert_count, parent_event_alert_count, child_alert_count,
  parent_alert_count, child_relation_count, parent_relation_count, child_first_name, child_mid_name, child_last_name,
  child_name_gen, parent_first_name, parent_mid_name, parent_last_name, parent_name_gen, rule_desc, likeness_score,
  relationship_score )
as
select r.conflict_id, r.parent_entity_id, r.child_entity_id, r.degree_of_sep, r.match_id, r.mm_rule_id, ees.matched_entity_id,
  case ees.matched_entity_id
    when r.parent_entity_id  then r.child_entity_id
    when r.child_entity_id then r.parent_entity_id
  end as inbound_entity_id,
(select event_alert_count from cog_event_summary where entity_id = r.child_entity_id) as child_event_alert_count,
(select event_alert_count from cog_event_summary where entity_id = r.parent_entity_id) as parent_event_alert_count,
/* inline the cog_releationship_summary view so that the where clause is applied immediately */
(SELECT  COALESCE((select count(*) from 
        vis_entity_conflicts where 
        conflict_stat not in ( 
            select act_code from 
            app_activity_codes where 
            act_status_code = 'INVALIDATED' and source_type = 'CONFLICT') 
            and conf_entity1 = e.entity_id or conf_entity2 = e.entity_id),0) 
    FROM entity e 
    where entity_id = r.child_entity_id) as child_alert_count,
/* inline the cog_releationship_summary view so that the where clause is applied immediately */
(SELECT COALESCE((select count(*) from  
        vis_entity_conflicts where 
        conflict_stat not in ( 
            select act_code from 
            app_activity_codes where 
            act_status_code = 'INVALIDATED' and source_type = 'CONFLICT') 
            and conf_entity1 = e.entity_id or conf_entity2 = e.entity_id),0) 
    FROM entity e 
    where entity_id = r.parent_entity_id) as parent_alert_count,
/* inline the cog_releationship_summary view so that the where clause is applied immediately */
(SELECT COALESCE((select count(*) from 
        VIS_RELATEDENTITIES vr where 
        vr.entity_id = e.entity_id or vr.relto_id = e.entity_id),0) 
    FROM entity e 
    where entity_id = r.child_entity_id) as child_relation_count,
/* inline the cog_releationship_summary view so that the where clause is applied immediately */
(SELECT COALESCE((select count(*) from 
    VIS_RELATEDENTITIES vr where 
    vr.entity_id = e.entity_id or vr.relto_id = e.entity_id),0) 
FROM entity e where 
    entity_id = r.parent_entity_id) as parent_relation_count,
(select first_name from best_name_conflict where er_id = r.match_id and conflict_entity_id = r.child_entity_id) as child_first_name,
(select mid_name from best_name_conflict where er_id = r.match_id and conflict_entity_id = r.child_entity_id) as child_mid_name,
(select last_name from best_name_conflict where er_id = r.match_id and conflict_entity_id = r.child_entity_id) as child_last_name,
(select name_gen from best_name_conflict where er_id = r.match_id and conflict_entity_id = r.child_entity_id) as child_name_gen,
(select first_name from best_name_conflict where er_id = r.match_id and conflict_entity_id = r.parent_entity_id) as parent_first_name,
(select mid_name from best_name_conflict where er_id = r.match_id and conflict_entity_id = r.parent_entity_id) as parent_mid_name,
(select last_name from best_name_conflict where er_id = r.match_id and conflict_entity_id = r.parent_entity_id) as parent_last_name,
(select name_gen from best_name_conflict where er_id = r.match_id and conflict_entity_id = r.parent_entity_id) as parent_name_gen,
(select mm_rule_desc from match_merge_rules where mm_rule_id = r.mm_rule_id) as rule_desc,
r.like_conf as likeness_score, r.rel_conf as relationship_score
from sep_conflict_rel r, sep_conflict s, er_entity_score ees
where r.conflict_id = s.conflict_id and
r.degree_of_sep <= s.degree_of_sep and
r.match_id = ees.er_id and
r.like_conf = ees.like_score and
(ees.matched_entity_id = r.child_entity_id or ees.matched_entity_id = r.parent_entity_id);